@@ -1,3 +1,8 @@
+use futures_signals::signal::{Signal, SignalExt, channel, Receiver};
+use std::marker::Unpin;
+use std::pin::Pin;
+use std::task::{Context, Poll};
+use std::rc::Rc;
 /*
  * This helper is to make it simpler to Box a signal-factory function
  * The use case is similar to where you might have a ReadOnlyMutable or Broadcaster
@@ -9,7 +14,7 @@
  *
  * Example:
  * pub struct Foo {
- *   pub active_signal: SignalFn<bool>,
+ *   pub active_signal: BoxSignalFn<bool>,
  * }
  *
  *
@@ -33,6 +38,15 @@ pub fn box_signal_fn<T, S: Signal<Item = T> + 'static>(f: impl Fn() -> S + 'stat
     })
 }
 
+/// Type alias for a Rc'd signal-factory (same idea as BoxSignalFn but cloneable)
+pub type RcSignalFn<T> = Rc<dyn Fn() -> Pin<Box<dyn Signal<Item = T>>>>;
+
+/// Helper to create Rc signal-factories (same idea as box_signal_fn but cloneable)
+pub fn rc_signal_fn<T, S: Signal<Item = T> + 'static>(f: impl Fn() -> S + 'static) -> RcSignalFn<T> {
+    Rc::new(move || {
+        Box::pin(f())
+    })
+}
 /*
  * These all generally solving the problem of where you need to return different types of signals
  * But don't want to Box it.
@@ -56,10 +70,6 @@ pub fn box_signal_fn<T, S: Signal<Item = T> + 'static>(f: impl Fn() -> S + 'stat
  * considered finished, the consumer must not poll again.
 */
 
-use futures_signals::signal::{Signal, SignalExt};
-use std::marker::Unpin;
-use std::pin::Pin;
-use std::task::{Context, Poll};
 
 /// If the provided signal is None,
 /// then a signal of the provided default
@@ -183,3 +193,73 @@ where
         }
     }
 }
+
+cfg_if::cfg_if! {
+    if #[cfg(all(feature = "awsm_web"))] {
+        use awsm_web::dom::resize::ResizeObserver;
+        use web_sys::{Element, DomRect};
+
+        pub struct DomRectSignal {
+            _observer: ResizeObserver,
+            receiver: Receiver<DomRect>,
+        }
+
+        impl Signal for DomRectSignal {
+            type Item = DomRect;
+
+            #[inline]
+            fn poll_change(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Option<Self::Item>> {
+                self.receiver.poll_change_unpin(cx)
+            }
+        }
+
+        pub fn dom_rect_signal(element: &Element) -> DomRectSignal {
+            let (sender, receiver) = channel(element.get_bounding_client_rect());
+
+            let observer = {
+                let element = element.clone();
+                
+                ResizeObserver::new_simple(move || {
+                    sender.send(element.get_bounding_client_rect()).unwrap();
+                })
+            };
+
+            observer.observe(&element);
+
+            DomRectSignal { _observer: observer, receiver }
+        }
+
+        pub struct DomRectMultiSignal {
+            _observer: ResizeObserver,
+            receiver: Receiver<Vec<DomRect>>,
+        }
+
+        impl Signal for DomRectMultiSignal {
+            type Item = Vec<DomRect>;
+
+            #[inline]
+            fn poll_change(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Option<Self::Item>> {
+                self.receiver.poll_change_unpin(cx)
+            }
+        }
+
+        pub fn dom_rect_multi_signal(elements: &[Element]) -> DomRectMultiSignal {
+            let init_sizes = elements.iter().map(|elem| elem.get_bounding_client_rect()).collect();
+
+            let (sender, receiver) = channel(init_sizes);
+
+            let observer = {
+                ResizeObserver::new(move |entries| {
+                    let sizes = entries.into_iter().map(|entry| entry.content_rect).collect();
+                    sender.send(sizes).unwrap();
+                }, None)
+            };
+
+            for element in elements {
+                observer.observe(&element);
+            }
+
+            DomRectMultiSignal { _observer: observer, receiver }
+        }
+    }
+}
